#!/usr/bin/env python3

#
# Code by Sarah Delgado, March 2020
#
# Code listens for touch input of the Act pad and compares it to the activation inputs saved at 'savedInput.txt'
# if a valid activation input is detected, the mapped usecase will be executed
#

from ActPad_mpr121_controller import mpr121
import os
import webbrowser
import requests
from pycmus import remote
import subprocess
import time

class ExecuteOnInteraction:
    def __init__(self,activationInputsPath,songPath):
        # load saved activation inputs from file
        activation_inputs,max_length_seq,max_length_sim = self.loadActivationInputs(activationInputsPath)
        print('ActivationStrings:',activation_inputs)
        
        self.song = songPath
        
        mpr121_controller = mpr121(True)
        self.quit = False # to terminate while
        #self.quit = True # to terminate while
        input_seq = "" # current sequential input
        input_sim = "" # current simultaneous input
        
        self.power_socket_1_ID = "5909" # ID of Delock WLAN Power Socket 1
        self.power_socket_2_ID = "5688" # ID of Delock WLAN Power Socket 2
        
        while(not self.quit):
            # listen for touch inputs
            input_seq,input_sim = self.touch_recognition(mpr121_controller,input_seq,input_sim)
            #print(input_seq,'\t',input_sim)
            
            for obj in activation_inputs:
                if activation_inputs[obj]['mode'] == 1:
                    # sequential mode
                    if obj in input_seq:
                        # Valid sequential activation input was detected
                        #print('Seq detected!!')
                        self.execute(activation_inputs[obj]['usecase'])
                        input_seq = ""
                        #quit = True
                        
                    if len(input_seq.split(' ')) >= 3*max_length_seq: input_seq = "" # reset input-string
                            
                if activation_inputs[obj]['mode'] == 2:
                    # simultaneous mode
                    if obj == input_sim:
                        # Valid simultaneous activation input was detected
                        #print('Sim detected!')
                        self.execute(activation_inputs[obj]['usecase'])
                        input_sim = "" # reset sim input-string
                        #quit = True
                        
                    if len(input_sim.split(' ')) >= max_length_sim: input_sim = "" # reset input-string
                
    def execute(self,usecase):
        print(usecase)
        if usecase == 'News':
            #os.system()
            #webbrowser.open('https://who.maps.arcgis.com/apps/opsdashboard/index.html#/c88e37cfc43b4ed3baf977d77e4a0667')
            webbrowser.open('https://www.bbc.com/news')
            
        if usecase == 'Turn Light On':
            self.togglePower(self.power_socket_2_ID)
            
        if usecase == 'Turn Monitor On':
            self.togglePower(self.power_socket_1_ID)
            
        if usecase == 'Preheat Coffe-Machine':
            self.togglePower(self.power_socket_1_ID)
        
        if usecase == 'Favorite Song':
            #Play your favorite song on cmus
            #subprocess.call('lxterminal --command "cmus"', shell=True)
            subprocess.Popen('lxterminal --command "cmus"', shell=True)
            print('cmus started')
            
            time.sleep(2)
            
            cmus = remote.PyCmus()
            print('connection to cmus established')
            #print(cmus.status())
            cmus.player_play_file(self.song)
            print('Song',self.song,'playing')
            #cmus.player_play()
        
        if usecase == 'Next Song':
            #Play the next song on cmus
            cmus = remote.PyCmus()
            print('connection to cmus established')
            #print(cmus.status())
            cmus.player_next()
            print('play next song')
    
    def togglePower(self,socket_ID):
        # Example Http GET ON/OFF: http://delock-5909/?m=1&o=1
        URL = "http://delock-"+socket_ID+"/"
        PARAMS = {'m':1,'o':1}
        r = requests.get(url = URL, params = PARAMS)
            
        # Process response to print it
        response_str = r.text.replace('{t}','').replace('</table>','').replace('></','').replace('><','')
        response_str = response_str[1:-1].split('>')[1]
        response_str = response_str.split('<')[0]
        print('Power',socket_ID,response_str)
        
    def loadActivationInputs(self,path):
        # load saved activation inputs from file
        max_length_seq = 0 #save maximum length of valid sequential activation codes
        max_length_sim = 0 #save maximum length of valid simultaneous activation codes
        
        with open(path, 'r') as file:
             file_content = file.readlines()
             file.close()
             
        activation_inputs = {} # save valid activation inputs loaded from file
        for line in file_content:
             line = line.replace('\n','')
             splitted = line.split(',')
             if len(splitted) == 3 and not 'Mode' in line:
                 activation_inputs[splitted[2]] = {
                                                 'usecase':splitted[0],
                                                 'mode':int(splitted[1])
                                                 }
                 
                 length = len(splitted[2].split(' ')) 
                 
                 if activation_inputs[splitted[2]]['mode'] == 1:
                    if length > max_length_seq: max_length_seq = length
                    
                 if activation_inputs[splitted[2]]['mode'] == 2:
                     if length > max_length_sim: max_length_sim = length
                     
                 
        return activation_inputs,max_length_seq,max_length_sim


    def touch_recognition(self,mpr121_controller,input_seq,input_sim):       
        (touched_pin,state) = mpr121_controller.get_touch_states()
        
        # Get sequential input
        if(state == 0 and not str(touched_pin) == 'None'):
                input_seq = input_seq+' '+str(touched_pin)
        
        # Get simultaneous input
        all_states = mpr121_controller.states
        currently_pressed_pins = ""
        i = 0
        for state in mpr121_controller.states:
            if state == 1:
                    currently_pressed_pins = currently_pressed_pins+' '+str(i)
            i = i+1
            
                    
        if len(input_sim.split(' ')) <= len(currently_pressed_pins.split(' ')):
                #change input only if the number of simultaneously touched keys increases
                input_sim = currently_pressed_pins
                   
        return input_seq,input_sim


path = 'savedInput.txt'
song = '/home/pi/Music/Paper_Navy_-_08_-_Swan_Song.mp3'
listener = ExecuteOnInteraction(path,song)

#usecases = ['Update on COVID-19','Turn Light On','Turn Monitor On','Preheat Coffe-Machine','Favorite Song','Next Song']
#listener.execute(usecases[4])
#time.sleep(10)
#listener.execute(usecases[5])

